/**
 * Classe oggetto che implementa un insieme di valori interi.
 * 
 * @author Luca Marchetti
 * @version 1.0
 */
public class Insieme implements IntegerSet {

	/**
	 * Vettore privato che memorizza gli elementi dell'insieme.
	 */
	private int[] elementi;
	
	/**
	 * Indice dell'ultimo valore salvato in <code>elementi</code> (-1 quando l'insieme e' vuoto).
	 */
	private int ultimoIndice = -1;

	/**
	 * Grandezza iniziale del vettore che salva gli elementi dell'insieme.
	 */
	private final int INITIAL_SIZE = 10;

	/**
	 * Crea un nuovo insieme.
	 * 
	 */
	public Insieme() {
		elementi = new int[INITIAL_SIZE];
	}
	
	/**
	 * Metodo che svuota l'insieme.
	 * 
	 */
	public void clear() {
		ultimoIndice = -1;
	}

	/**
	 * Metodo che restituisce il numero di elementi salvati.
	 * 
	 * @return numero elementi salvati
	 */
	public int size() {
		return ultimoIndice+1;
	}
		
	/**
	 * Metodo che rimuove dall'insieme l'elemento (se presente).
	 * 
	 * @param e elemento da rimuovere
	 */
	public void removeElement(int e) {
		int i;
		// cerco l'elemento da eliminare
		for (i = 0; i <= ultimoIndice; i++) {
			if (elementi[i] == e)
				break;
		}
		if (i <= ultimoIndice) {
			// se l'elemento da eliminare e' effettivamente nell'insieme, lo elimino con un opportuno shift a sinistra.
			for (int j = i; j < ultimoIndice; j++) {
				elementi[j] = elementi[j+1];
			}
			ultimoIndice -= 1;
		}
	}		
	
	/**
	 * Metodo che inserisce nell'insieme l'elemento (a patto che non sia gia' incluso).
	 * 
	 * @param e elemento da inserire
	 */
	public void insertElement(int e) {
		for (int i = 0; i <= ultimoIndice; i++) {
			if (elementi[i] == e)
				return; // se l'elemento e' gia' presente, non posso effettuare l'inserimento
		}
		if (ultimoIndice == elementi.length-1) {
			// se l'array che memorizza gli elementi dell'insieme e' pieno, ne creo uno piu' grande
			int[] nuoviElementi = new int[elementi.length+INITIAL_SIZE];
			for (int i = 0; i < elementi.length; i++)
				nuoviElementi[i] = elementi[i];
			elementi = nuoviElementi;
		}
		// aggiunta elemento
		elementi[ultimoIndice+1] = e;
		ultimoIndice += 1;
	}
	
	/**
	 * Metodo che restituisce gli elementi dell'insieme memorizzati in un vettore di interi.
	 * 
	 * @return array contenente gli elementi dell'insieme (<code>null</code> se l'insieme e' vuoto)
	 */
	public int[] getElements() {
		if (ultimoIndice < 0)
			return null; // ritorno null se l'insieme e' vuoto
		int[] vettElementi = new int[ultimoIndice+1]; // creo un vettore nuovo che sia lungo ESATTAMENTE come il numero di elementi da restituire
		for (int i = 0; i <= ultimoIndice; i++)
			vettElementi[i] = elementi[i];
		return vettElementi;
	}
}
